/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.apache.stratos.metadata.client.rest;

import org.apache.http.HttpResponse;
import org.apache.stratos.metadata.client.exception.RestClientException;

public interface RestClient {
    /**
     * Perform a HTTP POST request to @resourcePath with request body containing @payload converted to JSON.
     *
     * @param resourcePath Resource path to where is request is sent.
     * @param payload      request message body.
     * @return Response of the request.
     * @throws RestClientException
     */
    public HttpResponse doPost(String resourcePath, Object payload) throws RestClientException;

    /**
     * Perform a HTTP GET request to @resourcePath.
     *
     * @param resourcePath Resource path to where is request is sent.
     * @return Response of the request.
     * @throws RestClientException
     */
    public HttpResponse doGet(String resourcePath) throws RestClientException;

    /**
     * Perform a HTTP DELETE request to @resourcePath.
     *
     * @param resourcePath Resource path to where is request is sent.
     * @return Response of the request.
     * @throws RestClientException
     */
    public HttpResponse doDelete(String resourcePath) throws RestClientException;
}
